export declare type HistoryChangeCallbackRemovalHandler = () => void;
export interface HistoryEntry<TId, TData> {
    readonly id: TId;
    readonly ref: string;
    readonly data: TData | undefined;
}
export interface HistorySnapshot<TEntryId, TData> {
    readonly entries: readonly HistoryEntry<TEntryId, TData>[];
    readonly active: TEntryId;
}
export declare type HistoryChangeCallback<TEntryId, TData> = (snapshot: HistorySnapshot<TEntryId, TData>) => void;
declare abstract class History<TEntryId, TData> {
    private changeCallbackSet;
    abstract get snapshot(): HistorySnapshot<TEntryId, TData>;
    listen(callback: HistoryChangeCallback<TEntryId, TData>): HistoryChangeCallbackRemovalHandler;
    get ref(): string;
    get index(): number;
    get length(): number;
    abstract getHRefByRef(ref: string): string;
    abstract getRefByHRef(href: string): string;
    abstract back(): Promise<void>;
    abstract forward(): Promise<void>;
    abstract push(ref: string, data?: unknown): Promise<void>;
    abstract replace(ref: string, data?: unknown): Promise<void>;
    abstract restore(snapshot: HistorySnapshot<TEntryId, TData>, toEmitChange?: boolean): Promise<void>;
    protected emitChange(snapshot: HistorySnapshot<TEntryId, TData>): void;
}
export declare const AbstractHistory: typeof History;
export interface IHistory<TEntryId = unknown, TData = unknown> extends History<TEntryId, TData> {
}
export declare function isHistoryEntryEqual<TEntryId, TData>(x: HistoryEntry<TEntryId, TData>, y: HistoryEntry<TEntryId, TData>): boolean;
export declare function getActiveHistoryEntryIndex<TEntryId, TData>({ entries, active: activeId, }: HistorySnapshot<TEntryId, TData>): number;
export declare function getActiveHistoryEntry<TEntryId, TData>({ entries, active: activeId, }: HistorySnapshot<TEntryId, TData>): HistoryEntry<TEntryId, TData>;
export {};
