"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getActiveHistoryEntry = exports.getActiveHistoryEntryIndex = exports.isHistoryEntryEqual = exports.AbstractHistory = void 0;
class History {
    constructor() {
        Object.defineProperty(this, "changeCallbackSet", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Set()
        });
    }
    listen(callback) {
        callback(this.snapshot);
        this.changeCallbackSet.add(callback);
        return () => {
            this.changeCallbackSet.delete(callback);
        };
    }
    get ref() {
        return getActiveHistoryEntry(this.snapshot).ref;
    }
    get index() {
        return getActiveHistoryEntryIndex(this.snapshot);
    }
    get length() {
        return this.snapshot.entries.length;
    }
    emitChange(snapshot) {
        for (let callback of this.changeCallbackSet) {
            try {
                callback(snapshot);
            }
            catch (error) {
                console.error(error);
            }
        }
    }
}
exports.AbstractHistory = History;
function isHistoryEntryEqual(x, y) {
    return x.id === y.id && x.ref === y.ref && x.data === y.data;
}
exports.isHistoryEntryEqual = isHistoryEntryEqual;
function getActiveHistoryEntryIndex({ entries, active: activeId, }) {
    let index = entries.findIndex(entry => entry.id === activeId);
    if (index < 0) {
        console.error('Invalid history snapshot');
        index = entries.length - 1;
    }
    return index;
}
exports.getActiveHistoryEntryIndex = getActiveHistoryEntryIndex;
function getActiveHistoryEntry({ entries, active: activeId, }) {
    let entry = entries.find(entry => entry.id === activeId);
    if (!entry) {
        console.error('Invalid history snapshot');
        entry = entries[entries.length - 1];
    }
    return entry;
}
exports.getActiveHistoryEntry = getActiveHistoryEntry;
//# sourceMappingURL=history.js.map