"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MemoryHistory = void 0;
const history_1 = require("./history");
const SNAP_PROMISE = Promise.resolve();
class MemoryHistory extends history_1.AbstractHistory {
    constructor({ initialRef = '/', prefix = '' } = {}) {
        super();
        Object.defineProperty(this, "_snapshot", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lastId", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "prefix", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.prefix = prefix;
        if (!initialRef.startsWith('/')) {
            initialRef = `/${initialRef}`;
        }
        let id = 0;
        let data;
        let entries = [
            {
                id,
                ref: initialRef,
                data,
            },
        ];
        this._snapshot = {
            entries,
            active: id,
        };
        this.lastId = id;
    }
    get snapshot() {
        return this._snapshot;
    }
    getHRefByRef(ref) {
        return `${this.prefix}${ref}`;
    }
    getRefByHRef(href) {
        let prefix = this.prefix;
        return href.startsWith(prefix) ? href.slice(prefix.length) : href;
    }
    async back() {
        await SNAP_PROMISE;
        let snapshot = this._snapshot;
        let { entries } = snapshot;
        let index = (0, history_1.getActiveHistoryEntryIndex)(snapshot);
        if (index <= 0) {
            return;
        }
        snapshot = {
            entries,
            active: entries[index - 1].id,
        };
        this._snapshot = snapshot;
        this.emitChange(snapshot);
    }
    async forward() {
        await SNAP_PROMISE;
        let snapshot = this._snapshot;
        let { entries } = snapshot;
        let index = (0, history_1.getActiveHistoryEntryIndex)(snapshot);
        if (index >= entries.length - 1) {
            return;
        }
        snapshot = {
            entries,
            active: entries[index + 1].id,
        };
        this._snapshot = snapshot;
        this.emitChange(snapshot);
    }
    async push(ref, data) {
        let id = ++this.lastId;
        let snapshot = this._snapshot;
        let { entries } = snapshot;
        let index = (0, history_1.getActiveHistoryEntryIndex)(snapshot);
        snapshot = {
            entries: [...entries.slice(0, index + 1), { id, ref, data }],
            active: id,
        };
        this._snapshot = snapshot;
        this.emitChange(snapshot);
    }
    async replace(ref, data) {
        let snapshot = this._snapshot;
        let { entries, active: activeId } = snapshot;
        let index = (0, history_1.getActiveHistoryEntryIndex)(snapshot);
        snapshot = {
            entries: [
                ...entries.slice(0, index),
                { id: activeId, ref, data },
                ...entries.slice(index + 1),
            ],
            active: activeId,
        };
        this._snapshot = snapshot;
        this.emitChange(snapshot);
    }
    async restore(snapshot, toEmitChange = false) {
        this._snapshot = snapshot;
        if (toEmitChange) {
            this.emitChange(this._snapshot);
        }
    }
}
exports.MemoryHistory = MemoryHistory;
//# sourceMappingURL=memory-history.js.map