import { Dict, EmptyObjectPatch } from 'tslang';
import { IHistory } from '../history';
import { RouteBuilder } from '../route-builder';
import { Router, RouterNavigateOptions } from '../router';
import { RouteSource } from './route-match';
export declare type GeneralSegmentDict = Dict<string | undefined>;
export declare type GeneralQueryDict = Dict<string | undefined>;
export declare type GeneralParamDict = Dict<string | undefined>;
export declare type RouteMatchSharedToParamDict<TRouteMatchShared> = TRouteMatchShared extends RouteMatchShared<infer TParamDict> ? TParamDict : never;
export interface RouteMatchBuildOptions<TGroupName extends string> {
    /**
     * Whether to leave this match's group.
     */
    leave?: boolean;
    /**
     * Parallel route groups to leave.
     */
    leaves?: TGroupName | TGroupName[];
}
export interface RouteMatchNavigateOptions<TGroupName extends string> extends RouteMatchBuildOptions<TGroupName>, RouterNavigateOptions {
}
export interface RouteMatchSharedOptions {
    match: string | RegExp;
    query: Map<string, string | symbol | true>;
    group: string | undefined;
    metadata: object | undefined;
}
export declare abstract class RouteMatchShared<TParamDict extends GeneralParamDict = GeneralParamDict, TSpecificGroupName extends string | undefined = string | undefined, TGroupName extends string = string, TMetadata extends object = object> {
    /**
     * Name of this `RouteMatch`, correspondent to the field name of route
     * schema.
     */
    readonly $name: string;
    /**
     * Group of this `RouteMatch`, specified in the root route.
     */
    readonly $group: TSpecificGroupName | undefined;
    readonly $metadata: TMetadata;
    /**
     * Parent of this route match.
     */
    readonly $parent: RouteMatchShared | undefined;
    readonly $router: Router<TGroupName>;
    constructor(name: string, router: Router<TGroupName>, source: RouteSource, parent: RouteMatchShared | undefined, history: IHistory, { match, query, group, metadata }: RouteMatchSharedOptions);
    /**
     * A dictionary of the combination of query string and segments.
     */
    get $params(): TParamDict;
    /**
     * A reactive value indicates whether this route is exactly matched.
     */
    get $exact(): boolean;
    /**
     * A reactive value indicates whether this route is matched.
     */
    get $matched(): boolean;
    /**
     * Get the deepest matching descendant.
     */
    get $rest(): this;
    $(params?: Partial<TParamDict> & EmptyObjectPatch): RouteBuilder<TGroupName>;
    $ref(params?: Partial<TParamDict> & EmptyObjectPatch): string;
    $href(params?: Partial<TParamDict> & EmptyObjectPatch): string;
    $push(params?: Partial<TParamDict> & EmptyObjectPatch, { onComplete, ...options }?: RouteMatchNavigateOptions<TGroupName>): void;
    $replace(params?: Partial<TParamDict> & EmptyObjectPatch, { onComplete, ...options }?: RouteMatchNavigateOptions<TGroupName>): void;
}
