import { IAutorunOptions, IReactionDisposer, IReactionOptions, IReactionPublic } from 'mobx';
import { OmitValueOfKey, OmitValueWithType } from 'tslang';
import { IHistory } from '../history';
import { Router } from '../router';
import { NextRouteMatch } from './next-route-match';
import { GeneralParamDict, RouteMatchShared, RouteMatchSharedOptions } from './route-match-shared';
export interface RouteUpdateCallbackData {
    descendants: boolean;
}
/**
 * Route before enter callback.
 * @return Return `true` or `undefined` to do nothing; return `false` to revert
 * this history change; return full path to redirect.
 */
export declare type RouteBeforeEnterCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<boolean | void> | boolean | void;
/**
 * Route before update callback.
 * @return Return `true` or `undefined` to do nothing; return `false` to revert
 * this history change; return full path to redirect.
 */
export declare type RouteBeforeUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next'], data: RouteUpdateCallbackData) => Promise<boolean | void> | boolean | void;
export interface RouteBeforeUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteBeforeUpdateEntry<TRouteMatch extends RouteMatch = RouteMatch> {
    callback: RouteBeforeUpdateCallback<TRouteMatch>;
    options?: RouteBeforeUpdateOptions;
}
/**
 * Route before leave callback.
 * @return Return `true` or `undefined` to do nothing; return `false` to revert
 * this history change.
 */
export declare type RouteBeforeLeaveCallback = () => Promise<boolean | void> | boolean | void;
export declare type RouteWillEnterCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<void> | void;
export declare type RouteWillUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next'], data: RouteUpdateCallbackData) => Promise<void> | void;
export interface RouteWillUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteWillUpdateEntry<TRouteMatch extends RouteMatch = RouteMatch> {
    callback: RouteWillUpdateCallback<TRouteMatch>;
    options?: RouteWillUpdateOptions;
}
export declare type RouteWillLeaveCallback = () => Promise<void> | void;
export declare type RouteEnterCallback = () => void;
export declare type RouteUpdateCallback = (data: RouteUpdateCallbackData) => void;
export interface RouteUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteUpdateEntry {
    callback: RouteUpdateCallback;
    options?: RouteUpdateOptions;
}
export declare type RouteLeaveCallback = () => void;
export declare type RouteAfterEnterCallback = () => void;
/**
 * Route after update callback.
 */
export declare type RouteAfterUpdateCallback = (data: RouteUpdateCallbackData) => void;
export interface RouteAfterUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteAfterUpdateEntry {
    options?: RouteAfterUpdateOptions;
    callback: RouteAfterUpdateCallback;
}
export declare type RouteAfterLeaveCallback = () => void;
export declare type RouteReactiveDisposer = IReactionDisposer;
export declare type RouteAutorunView = (reaction: IReactionPublic) => void;
export declare type RouteAutorunOptions = IAutorunOptions;
export declare type RouteReactionExpression<T> = (reaction: IReactionPublic) => T;
export declare type RouteReactionEffect<T> = (value: T, previousValue: T, reaction: IReactionPublic) => void;
export declare type RouteReactionOptions<T, FireImmediately extends boolean = boolean> = IReactionOptions<T, FireImmediately> | undefined;
export declare type RouteHookRemovalCallback = () => void;
export declare type RouteBeforeEnterOrUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<boolean | void> | boolean | void;
export declare type RouteWillEnterOrUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<void> | void;
export declare type RouteAfterEnterOrUpdateCallback = () => void;
export declare type RouteServiceFactory<TRouteMatch extends RouteMatch> = (match: TRouteMatch) => IRouteService<TRouteMatch> | Promise<IRouteService<TRouteMatch>>;
export declare type IRouteService<TRouteMatch extends RouteMatch = RouteMatch> = {
    beforeEnter?: RouteBeforeEnterCallback<TRouteMatch>;
    willEnter?: RouteWillEnterCallback;
    enter?: RouteEnterCallback;
    afterEnter?: RouteAfterEnterCallback;
    beforeUpdate?: RouteBeforeUpdateCallback<TRouteMatch>;
    willUpdate?: RouteWillUpdateCallback<TRouteMatch>;
    update?: RouteUpdateCallback;
    afterUpdate?: RouteAfterUpdateCallback;
    beforeLeave?: RouteBeforeLeaveCallback;
    willLeave?: RouteWillLeaveCallback;
    leave?: RouteLeaveCallback;
    afterLeave?: RouteAfterLeaveCallback;
} & RouteServiceExtension<TRouteMatch>;
export declare type RouteServiceExtension<TRouteMatch extends RouteMatch> = OmitValueWithType<OmitValueOfKey<TRouteMatch, keyof RouteMatch>, RouteMatch, false>;
export interface RouteMatchParallelOptions<TGroupName extends string> {
    groups?: TGroupName[];
    matches?: RouteMatch[];
}
export interface RouteMatchEntry {
    match: RouteMatch;
    exact: boolean;
    segment: string;
    rest: string;
}
export interface RouteSourceQuery {
    id: string | symbol | true;
    value: string;
}
export interface RouteSource {
    groupToMatchToMatchEntryMapMap: Map<string | undefined, Map<RouteMatch, RouteMatchEntry>>;
    queryMap: Map<string, RouteSourceQuery>;
    pathMap: Map<string | undefined, string>;
}
export interface RouteMatchOptions extends RouteMatchSharedOptions {
    exact: boolean | string;
}
export declare class RouteMatch<TParamDict extends GeneralParamDict = GeneralParamDict, TNextRouteMatch extends NextRouteMatch<TParamDict> = NextRouteMatch<TParamDict>, TSpecificGroupName extends string | undefined = string | undefined, TGroupName extends string = string, TMetadata extends object = object> extends RouteMatchShared<TParamDict, TSpecificGroupName, TGroupName, TMetadata> {
    readonly $parent: RouteMatch | undefined;
    readonly $next: TNextRouteMatch;
    constructor(name: string, router: Router<TGroupName>, source: RouteSource, parent: RouteMatch | undefined, extension: object | undefined, history: IHistory, { exact, ...sharedOptions }: RouteMatchOptions);
    $beforeEnter(callback: RouteBeforeEnterCallback<this>): RouteHookRemovalCallback;
    $beforeUpdate(callback: RouteBeforeUpdateCallback<this>, options?: RouteBeforeUpdateOptions): RouteHookRemovalCallback;
    $beforeLeave(callback: RouteBeforeLeaveCallback): RouteHookRemovalCallback;
    $willEnter(callback: RouteWillEnterCallback): RouteHookRemovalCallback;
    $willUpdate(callback: RouteWillUpdateCallback, options?: RouteWillUpdateOptions): RouteHookRemovalCallback;
    $willLeave(callback: RouteWillLeaveCallback): RouteHookRemovalCallback;
    $enter(callback: RouteEnterCallback): RouteHookRemovalCallback;
    $update(callback: RouteUpdateCallback, options?: RouteUpdateOptions): RouteHookRemovalCallback;
    $leave(callback: RouteLeaveCallback): RouteHookRemovalCallback;
    $afterEnter(callback: RouteAfterEnterCallback): RouteHookRemovalCallback;
    $afterUpdate(callback: RouteAfterUpdateCallback, options?: RouteAfterUpdateOptions): RouteHookRemovalCallback;
    $afterLeave(callback: RouteAfterLeaveCallback): RouteHookRemovalCallback;
    $autorun(view: RouteAutorunView, options?: RouteAutorunOptions): RouteHookRemovalCallback;
    $reaction<T>(expression: RouteReactionExpression<T>, effect: RouteReactionEffect<T>, options?: RouteReactionOptions<T>): RouteHookRemovalCallback;
    $beforeEnterOrUpdate(callback: RouteBeforeEnterOrUpdateCallback<this>, beforeUpdateOptions?: RouteBeforeUpdateOptions): RouteHookRemovalCallback;
    $willEnterOrUpdate(callback: RouteWillEnterOrUpdateCallback<this>, willUpdateOptions?: RouteWillUpdateOptions): RouteHookRemovalCallback;
    $afterEnterOrUpdate(callback: RouteAfterEnterOrUpdateCallback, afterUpdateOptions?: RouteAfterUpdateOptions): RouteHookRemovalCallback;
    $service(factory: RouteServiceFactory<this>): this;
    $parallel(options: RouteMatchParallelOptions<TGroupName>): void;
    static SEGMENT: RegExp;
    static REST: RegExp;
}
