import { Dict, EmptyObjectPatch } from 'tslang';
import { HistorySnapshot, IHistory } from './history';
import { RouteBuilder } from './route-builder';
import { NextRouteMatch, RouteMatch, RouteMatchShared, RouteMatchSharedToParamDict } from './route-match';
import { RootRouteSchema } from './schema';
export declare type SegmentMatcherCallback = (key: string) => string;
declare type RouteQuerySchemaType<TRouteSchema> = TRouteSchema extends {
    $query: infer TQuerySchema;
} ? TQuerySchema : {};
declare type FilterRouteMatchNonStringSegment<TRouteSchema, T> = TRouteSchema extends {
    $match: infer TMatch;
} ? TMatch extends string ? never : T : never;
interface RouteSchemaChildrenSection<TRouteSchemaDict> {
    $children: TRouteSchemaDict;
}
declare type NestedRouteSchemaDictType<TRouteSchema> = TRouteSchema extends RouteSchemaChildrenSection<infer TNestedRouteSchemaDict> ? TNestedRouteSchemaDict : {};
interface RouteSchemaExtensionSection<TRouteMatchExtension> {
    $extension: TRouteMatchExtension;
}
interface RouteSchemaMetadataSection<TMetadata> {
    $metadata: TMetadata;
}
declare type RouteMatchMetadataType<TRouteSchema, TUpperMetadata> = TRouteSchema extends RouteSchemaMetadataSection<infer TMetadata> ? TMetadata & TUpperMetadata : TUpperMetadata;
declare type RouteMatchExtensionType<TRouteSchema> = TRouteSchema extends RouteSchemaExtensionSection<infer TRouteMatchExtension> ? TRouteMatchExtension : {};
declare type RouteMatchSegmentType<TRouteSchemaDict, TSegmentKey extends string, TQueryKey extends string, TSpecificGroupName extends string | undefined, TGroupName extends string, TMetadata extends object> = {
    [K in Extract<keyof TRouteSchemaDict, string>]: RouteMatchType<TRouteSchemaDict[K], TSegmentKey | FilterRouteMatchNonStringSegment<TRouteSchemaDict[K], K>, TQueryKey | Extract<keyof RouteQuerySchemaType<TRouteSchemaDict[K]>, string>, TSpecificGroupName, TGroupName, TMetadata>;
};
declare type __RouteMatchType<TRouteSchema, TSegmentKey extends string, TQueryKey extends string, TSpecificGroupName extends string | undefined, TGroupName extends string, TParamDict extends Dict<string | undefined>, TMetadata extends object> = RouteMatch<TParamDict, __NextRouteMatchType<TRouteSchema, TSegmentKey, TQueryKey, TSpecificGroupName, TGroupName, TParamDict, TMetadata>, TSpecificGroupName, TGroupName, RouteMatchMetadataType<TRouteSchema, TMetadata>> & RouteMatchSegmentType<NestedRouteSchemaDictType<TRouteSchema>, TSegmentKey, TQueryKey, TSpecificGroupName, TGroupName, RouteMatchMetadataType<TRouteSchema, TMetadata>> & RouteMatchExtensionType<TRouteSchema>;
export declare type RouteMatchType<TRouteSchema, TSegmentKey extends string, TQueryKey extends string, TSpecificGroupName extends string | undefined, TGroupName extends string, TMetadata extends object> = __RouteMatchType<TRouteSchema, TSegmentKey, TQueryKey, TSpecificGroupName, TGroupName, Record<TQueryKey, string | undefined> & Record<TSegmentKey, string>, TMetadata>;
declare type NextRouteMatchSegmentType<TRouteSchemaDict, TSegmentKey extends string, TQueryKey extends string, TSpecificGroupName extends string | undefined, TGroupName extends string, TMetadata extends object> = {
    [K in Extract<keyof TRouteSchemaDict, string>]: NextRouteMatchType<TRouteSchemaDict[K], TSegmentKey | FilterRouteMatchNonStringSegment<TRouteSchemaDict[K], K>, TQueryKey | Extract<keyof RouteQuerySchemaType<TRouteSchemaDict[K]>, string>, TSpecificGroupName, TGroupName, TMetadata>;
};
declare type __NextRouteMatchType<TRouteSchema, TSegmentKey extends string, TQueryKey extends string, TSpecificGroupName extends string | undefined, TGroupName extends string, TParamDict extends Dict<string | undefined>, TMetadata extends object> = NextRouteMatch<TParamDict, TSpecificGroupName, TGroupName, RouteMatchMetadataType<TRouteSchema, TMetadata>> & NextRouteMatchSegmentType<NestedRouteSchemaDictType<TRouteSchema>, TSegmentKey, TQueryKey, TSpecificGroupName, TGroupName, RouteMatchMetadataType<TRouteSchema, TMetadata>>;
declare type NextRouteMatchType<TRouteSchema, TSegmentKey extends string, TQueryKey extends string, TSpecificGroupName extends string | undefined, TGroupName extends string, TMetadata extends object> = __NextRouteMatchType<TRouteSchema, TSegmentKey, TQueryKey, TSpecificGroupName, TGroupName, Record<TQueryKey, string | undefined> & Record<TSegmentKey, string>, TMetadata>;
export declare type RootRouteMatchType<TRouteSchema, TSpecificGroupName extends string | undefined, TGroupName extends string, TMetadata extends object = {}> = RouteMatchType<TRouteSchema, never, Extract<keyof RouteQuerySchemaType<TRouteSchema>, string>, TSpecificGroupName, TGroupName, TMetadata>;
export declare type RouterOnLeave = (path: string) => void;
export declare type RouterOnNavigateComplete = () => void;
export interface RouterHistoryEntryData {
    navigateCompleteListener?: RouterOnNavigateComplete;
}
export interface RouterOptions {
    /**
     * A function to perform default schema field name to segment string
     * transformation.
     */
    segmentMatcher?: SegmentMatcherCallback;
}
export interface RouterNavigateOptions {
    /**
     * The callback that will be called after a route completed (after all the hooks).
     */
    onComplete?: RouterOnNavigateComplete;
}
export declare type RouterHistory = IHistory<unknown, RouterHistoryEntryData>;
export declare type RouterHistorySnapshot = HistorySnapshot<unknown, RouterHistoryEntryData>;
export declare class Router<TGroupName extends string = string> {
    constructor(history: RouterHistory, { segmentMatcher }?: RouterOptions);
    get $routing(): boolean;
    get $snapshot(): RouterHistorySnapshot | undefined;
    get $nextSnapshot(): RouterHistorySnapshot | undefined;
    get $current(): RouteBuilder<TGroupName>;
    get $next(): RouteBuilder<TGroupName>;
    get $groups(): TGroupName[];
    $route<TPrimaryRouteSchema extends RootRouteSchema>(schema: TPrimaryRouteSchema): RootRouteMatchType<TPrimaryRouteSchema, undefined, TGroupName>;
    $route<TRouteSchema extends RootRouteSchema, TSpecificGroupName extends TGroupName>(group: TSpecificGroupName, schema: TRouteSchema): RootRouteMatchType<TRouteSchema, TSpecificGroupName, TGroupName>;
    $ref(): string;
    $href(): string;
    $<TRouteMatchShared extends RouteMatchShared>(route: TRouteMatchShared, params?: Partial<RouteMatchSharedToParamDict<TRouteMatchShared>> & EmptyObjectPatch): RouteBuilder<TGroupName>;
    $(part: string): RouteBuilder<TGroupName>;
    $scratch(): RouteBuilder<TGroupName>;
    $push(ref: string, options?: RouterNavigateOptions): void;
    $replace(ref: string, options?: RouterNavigateOptions): void;
}
export {};
